import click

from app.add import add_up_timesheet


@click.command()
@click.argument("filename", type=click.Path(exists=True))
def cli(filename):
    with open(filename) as fin:
        timesheet_string = fin.read()
    click.echo(add_up_timesheet(timesheet_string))



from datetime import datetime, date, time
from dateutil import parser
from typing import List, Union, Tuple

from types_ import Minutes


def subtract_times(first: time, second: time) -> Minutes:
    return int(
        (
            datetime.combine(datetime(1, 1, 1, 0, 0, 0), second)
            - datetime.combine(date.today(), first)
        ).seconds
        / 60
    )


def add_up_timesheet(timesheet_string: str) -> Minutes:
    total_minutes = 0

    for line in timesheet_string.split("\n"):
        time_range_strings = find_time_ranges(line)
        if time_range_strings:
            for time_range_string in time_range_strings:
                start_string, end_string = split_time_range_string(time_range_string)
                start, end = map(parse_time, [start_string, end_string])
                total_minutes += subtract_times(start, end)
    return total_minutes


def find_time_ranges(timesheet_line: str) -> Union[list, List[str]]:
    remove_hashes_and_leading_trailing_spaces = timesheet_line.replace("#", "").strip()
    try:
        parse_time(remove_hashes_and_leading_trailing_spaces)
    except ValueError:
        return []
    else:
        if ":" not in remove_hashes_and_leading_trailing_spaces:
            return []
        return remove_hashes_and_leading_trailing_spaces.split(", ")


def parse_time(time_string: str) -> time:
    return parser.parse(time_string).time()


def split_time_range_string(time_range_string) -> Tuple[str]:
    return time_range_string.split("-")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

test_requirements = ["pytest"]

setup(
    author="Zev Averbach",
    author_email="zev@averba.ch",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    description="A quick way to add up time entries in a markdown document.",
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="markdown timesheet",
    name="markdown_timesheet",
    packages=find_packages(include=["app"]),
    test_suite="tests",
    tests_require=test_requirements,
    entry_points="""
        [console_scripts]
        add=app.cli:cli
    """,
    url="https://github.com/zevaverbach/markdown_timesheet",
    version="0.01",
    zip_safe=False,
)
